#!/usr/bin/env python3
# -*- encoding: utf-8 -*-
"""
@File    :   export_pipeline.py
"""

from ray.data.read_api import read_datasource
import ray
import sys
import os

__work_dir__ = os.path.dirname(os.path.dirname(os.path.dirname(os.path.dirname(__file__))))
sys.path.insert(0, __work_dir__)
from windmillartifactv1.client.artifact_client import ArtifactClient
from windmillcomputev1.filesystem import init_py_filesystem

from vistudio.annotation.datasink.filename_provider import MultiFilenameProvider
from sdk.python.vistudio.annotation.config.arg_parser import ArgParser
from windmilltrainingv1.client.training_api_dataset import DatasetName
from base_export_pipeline import BaseExportPipeline
from vistudio.annotation.processor.exporter.coco.coco_preprocessor import CocoFormatPreprocessor, CocoMergePreprocessor
import argparse
import logit

logit.base_logger.setup_logger({})


class ExportCocoPipeline(BaseExportPipeline):
    """
    exporter coco pipeline
    """

    def __init__(self, args):
        super().__init__(args)
        self._py_fs = init_py_filesystem(self.config.filesystem)
        self.artifact_client = ArtifactClient(
            endpoint=self.config.windmill_endpoint,
            ak=self.config.windmill_ak,
            sk=self.config.windmill_sk
        )

    def run(self, parallelism: int = 10):
        """
        pipeline_coco
        :return:
        """
        # step 1: datasource
        ds = read_datasource(self.datasource, parallelism=parallelism)
        logit.info("read data from mongo.dataset count = {}".format(ds.count()))
        if ds.count() <= 0:
            return

        coco_format_preprocessor = CocoFormatPreprocessor(labels=self.labels, merge_labels=self.merge_labels)
        coco_merge_preprocessor = CocoMergePreprocessor(labels=self.labels)
        format_ds = coco_format_preprocessor.transform(ds)
        logit.info("format dataset.dataset count = {}".format(format_ds.count()))
        merge_ds = coco_merge_preprocessor.fit(format_ds).stats_
        logit.info("merge dataset.dataset count = {}".format(merge_ds.count()))

        # setp 3: writer
        dataset_name = DatasetName(workspace_id=self.dataset.get('workspaceID'),
                                   project_name=self.dataset.get('projectName'),
                                   local_name=self.dataset.get('localName'))
        object_name = dataset_name.get_name()
        location_resp = self.artifact_client.create_location(
            object_name=object_name
        )
        path = location_resp.location[len("s3://"):].strip("/")
        logit.info("create windmill location. location= {}".format(path))

        annotation_filename_provider = MultiFilenameProvider(file_name="annotation.json")
        merge_ds.write_json(path=path,
                            filesystem=self._py_fs,
                            filename_provider=annotation_filename_provider)
        label_txt_full_path = os.path.join(location_resp.location, "labels.txt")
        self.save_label_file(file_path=label_txt_full_path, labels=self.labels, start_index=0)
        # create dataset
        self.create_dataset(location=location_resp.location,
                            annotation_format=self.annotation_format,
                            dataset=self.dataset)


def run(args):
    """
    main
    :param args:
    :return:
    """
    pipeline = ExportCocoPipeline(args)
    pipeline.run()


if __name__ == "__main__":
    arg_parser = ArgParser(kind='Export')
    args = arg_parser.parse_args()
    run(args)